package org.openxma.dsl.platform.dao.impl;

import java.beans.PropertyDescriptor;
import java.util.Collections;

import org.openxma.dsl.platform.dao.FilterExpression;
import org.openxma.dsl.platform.dao.Filterable;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.InvalidPropertyException;


/**
 * TODO: pls document me
 */
public class ExampleExpression extends
		CompositeExpression {

	private static final String PROPERTY_MAX_RESULTS = "maxResults";
	private static final String PROPERTY_FIRST_RESULT = "firstResult";
	private static final String SUFFIX_EQUALS_IGNORE_CASE = "IgnoreCase";
	private static final String SUFFIX_LIKE = "Like";
	private final BeanWrapperImpl beanWrapper;

	public ExampleExpression(Object bean) {
		super(Collections.<FilterExpression>emptyList());
		beanWrapper = new BeanWrapperImpl(bean);
	}

	public Object applyTo(Filterable filterable) {
		return createFilterSpecificationFromBean().applyTo(filterable);
	}

	private FilterExpression createFilterSpecificationFromBean() {

		PropertyDescriptor[] propertyDescriptors = beanWrapper
				.getPropertyDescriptors();

		for (int i = 0; i < propertyDescriptors.length; i++) {
			PropertyDescriptor propertyDescriptor = propertyDescriptors[i];

			String propertyName = propertyDescriptor.getName();
			Object propertyValue = beanWrapper.getPropertyValue(propertyName);

			if (!new ReservedPropertyNameFilter().isFiltered(propertyName)
					&& null != propertyValue) {

				boolean ignoreCase = isEqualsIgnoreCaseProperty(propertyName);

				FilterExpression comparisonFilterSpecification = isLikeProperty(propertyName) ? ComparisonExpression
						.createLike(propertyName, propertyValue, ignoreCase)
						: ComparisonExpression.createEquals(
								propertyName, propertyValue, ignoreCase);

				addFilterSpecification(comparisonFilterSpecification);

			}
		}

		// test if LimitSpecification applies
		if (!getFilterExpressions().isEmpty()
				&& beanWrapper
						.getPropertyDescriptor(PROPERTY_FIRST_RESULT) != null
				&& beanWrapper.getPropertyDescriptor(PROPERTY_MAX_RESULTS) != null) {
			Integer firstResult = (Integer) beanWrapper
					.getPropertyValue(PROPERTY_FIRST_RESULT);
			Integer maxResults = (Integer) beanWrapper
					.getPropertyValue(PROPERTY_MAX_RESULTS);

			if (firstResult != null && maxResults != null) {
				addFilterSpecification(new ResultLimitExpression(
						firstResult, maxResults));
			}
		}

		return createAnd(getFilterExpressions());
	}

	private boolean isEqualsIgnoreCaseProperty(String propertyName) {
		try {
			PropertyDescriptor equalsIgnoreCasePropertyDescriptor = beanWrapper
					.getPropertyDescriptor(propertyName
							+ SUFFIX_EQUALS_IGNORE_CASE);
			return isPropertyValueEnabled(equalsIgnoreCasePropertyDescriptor);
		} catch (InvalidPropertyException invalidPropertyException) {
		}
		return false;
	}

	private boolean isLikeProperty(String propertyName) {
		try {
			PropertyDescriptor likePropertyDescriptor = beanWrapper
					.getPropertyDescriptor(propertyName + SUFFIX_LIKE);
			return isPropertyValueEnabled(likePropertyDescriptor);
		} catch (InvalidPropertyException invalidPropertyException) {
		}
		return false;
	}

	private boolean isPropertyValueEnabled(PropertyDescriptor propertyDescriptor) {
		Object propertyValue = beanWrapper
				.getPropertyValue(propertyDescriptor.getName());
		return Boolean.TRUE.equals(propertyValue);
	}

	private interface PropertyNameFilter {
		boolean isFiltered(String propertyName);
	}

	private class ReservedPropertyNameFilter implements PropertyNameFilter {

		public boolean isFiltered(String propertyName) {
			return propertyName.endsWith(SUFFIX_EQUALS_IGNORE_CASE)
					|| propertyName.endsWith(SUFFIX_LIKE)
					|| propertyName.equals(PROPERTY_FIRST_RESULT)
					|| propertyName.equals(PROPERTY_MAX_RESULTS)
					|| propertyName.equals("class");
		}

	}

}
