/*
 * @(#) $Id:  $
 *
 * Copyright 2009/2010 by sIT Solutions,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package org.openxma.dsl.platform.exceptions;

import org.openxma.dsl.platform.i18n.MessageResolvable;


/**
 * 'Generic' unchecked exception class using an error code to communicate the error that occurred.
 * Should be used for classical runtime exceptions as well as for application exceptions.<br/>
 * Implements the interface {@link MessageResolvable} so that an exception can be resolved to an
 * error message.<br/>
 * The error code is supposed to be used as key in resource bundles, the arguments will be used as
 * arguments for the message.
 * 
 */
public abstract class ErrorCodedException extends RuntimeException implements MessageResolvable {

    private static final long serialVersionUID = 1L;

    /**
     * message arguments
     */
    private Object[] arguments;
    /**
     * The error code communicates the type of exception,
     * also serves as key of the {@link MessageResolvable}.
     */
    private String errorCode;

    public ErrorCodedException(String errorCode, String defaultMessage) {
        super(defaultMessage);
        this.errorCode = errorCode;
    }

    public ErrorCodedException(String errorCode, Object[] arguments, String defaultMessage) {
        super(defaultMessage);
        this.arguments = arguments;
        this.errorCode = errorCode;
    }

    public ErrorCodedException(String errorCode, Throwable detail, String defaultMessage) {
        super(defaultMessage, detail);
        this.errorCode = errorCode;
    }
    
    public ErrorCodedException(String errorCode, Object[] arguments, Throwable detail, String defaultMessage) {
        super(defaultMessage, detail);
        this.arguments = arguments;
        this.errorCode = errorCode;
    }
    
    public ErrorCodedException(Throwable detail, String defaultMessage) {
        super(defaultMessage, detail);
    }
    
    public ErrorCodedException(Throwable detail) {
        super(detail);
    }
    
    public String getErrorCode() {
        return this.errorCode;
    }
    
    public Object[] getArguments() {
        return this.arguments;
    }
    
    public String getKey() {
        return getErrorCode();
    }

    public String getDefaultMessage() {
        return getMessage();
    }
    
}
