package org.openxma.dsl.platform.validation;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.util.Assert;
import org.springframework.util.ClassUtils;

public abstract class BaseValidators {
    private static final Map<Class<?>, Object> PRIMITIVE_DEFAULT_VALUES = new HashMap<Class<?>, Object>();
    static {
        PRIMITIVE_DEFAULT_VALUES.put(Integer.class, new Integer(0));
        PRIMITIVE_DEFAULT_VALUES.put(Boolean.class, Boolean.FALSE);
        PRIMITIVE_DEFAULT_VALUES.put(Short.class, new Short((short) 0));
        PRIMITIVE_DEFAULT_VALUES.put(Byte.class, new Byte((byte) 0));
        PRIMITIVE_DEFAULT_VALUES.put(Character.class, new Character((char) 0));
        PRIMITIVE_DEFAULT_VALUES.put(Long.class, new Long(0));
        PRIMITIVE_DEFAULT_VALUES.put(Double.class, new Double(0));
        PRIMITIVE_DEFAULT_VALUES.put(Float.class, new Float(0));
    };
    
    /**
     * 
     * @param object
     *            the object containing the given property
     * @param property
     *            name of the property value to return
     * @return the value of the given property or the primitive default value in
     *         case of null valued wrapper type
     */
    public <T> T getValue(Object object, String property, Class<T> propertyType) {
        Assert.notNull(object, "parameter 'object' must not be null");
        Assert.notNull(property, "parameter 'property' must not be null");
        BeanWrapper beanWrapper = new BeanWrapperImpl(object);
        Object value = beanWrapper.getPropertyValue(property);
        if (value == null && ClassUtils.isPrimitiveWrapper(propertyType)) {
            value = PRIMITIVE_DEFAULT_VALUES.get(propertyType);
        }
        return propertyType.cast(value);
    }
}
