package org.openxma.dsl.platform.validation.constraints;

import static java.lang.annotation.ElementType.ANNOTATION_TYPE;
import static java.lang.annotation.ElementType.CONSTRUCTOR;
import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import javax.validation.Constraint;
import javax.validation.Payload;

import org.openxma.dsl.platform.validation.constraints.impl.FormatValidator;

import at.spardat.enterprise.fmt.IFmt;

/**
 * Validates the annotated element using <code>{@link FormatValidator}</code>
 */
@Target({ METHOD, FIELD, ANNOTATION_TYPE, CONSTRUCTOR, PARAMETER })
@Retention(RUNTIME)
@Documented
@Constraint(validatedBy = {FormatValidator.class})
public @interface ValidFormat {
    String message() default "{org.openxma.dsl.platform.validation.constraints.ValidFormat.message}";

    Class<?>[] groups() default { };

    Class<? extends Payload>[] payload() default {};
    
    /**
     * @return custom format string.
     */
    String format() default "";

    /**
     * @return user defined validator to validate with.
     */
    Class<? extends IFmt>[] validatorClass() default {};

    /**
     * @return parameter for validation.
     */
    String value() default "";

    /**
     * Defines several <code>@ValidCustomFormat</code> annotations on the same element
     * @see ValidFormat
     */
    @Target({ METHOD, FIELD, ANNOTATION_TYPE, CONSTRUCTOR, PARAMETER })
    @Retention(RUNTIME)
    @Documented
    @interface List {
        ValidFormat[] value();
    }
    
}
