package org.openxma.dsl.platform.validation.constraints.impl;

import java.util.Locale;

import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;

import org.openxma.dsl.platform.validation.constraints.ValidFormat;
import org.springframework.context.i18n.LocaleContextHolder;

import at.spardat.enterprise.fmt.FmtFactory;
import at.spardat.enterprise.fmt.FmtParseException;
import at.spardat.enterprise.fmt.IFmt;

public class FormatValidator implements ConstraintValidator<ValidFormat, Object>{

    private IFmt format;
    
    public void initialize(ValidFormat constraintAnnotation) {
            Class<? extends IFmt>[] validator = constraintAnnotation.validatorClass();
            if (validator == null || validator.length == 0) {
                try {
                    format = FmtFactory.create(constraintAnnotation.format(), getLocale());
                } catch (RuntimeException e) {
                    throw new IllegalArgumentException("Illegal format specification: \"" + constraintAnnotation.format() + "\"", e);
                }
            } else if (validator.length == 1){
                try {
                    format = validator[0].newInstance();
                } catch (Exception e) {
                    throw new IllegalArgumentException("Unable to instantiate validator: " + validator[0], e);
                }
            } else {
                throw new IllegalArgumentException("Unable to instantiate validator, max. 1 format class allowed");
            }
        
    }

    public boolean isValid(Object value, ConstraintValidatorContext context) {
        if (format == null) {
            throw new IllegalStateException("Validator not initialized");
        }
        try {
            format.parse(value == null ? null : value.toString());
        } catch (FmtParseException fmtParseException) {
            return false;
        }
        return true;
    }
    
    /**
     * @return the default <code>Locale</code>
     */
    protected Locale getLocale() {
        return LocaleContextHolder.getLocale();
    }
    

}
